/**
 * \file bioritmo.h
 * \author Flameeyes <dgp85@users.sourceforge.net>
 *
 * \brief File di intestazione per la classe bioritmo
 *
 * Includere questo file per poter ususare la classe bioritmo per effettuare il
 * calcolo dei bioritmi data la data di nascita e quella di cui si vuole sapere
 * il bioritmo.
 *
 */
#ifndef T_BIORITMO_H
#define T_BIORITMO_H

#include <ctime>

#include <qdatetime.h>

/**
 * \class bioritmo bioritmo.h
 * \brief Classe per il calcolo dei bioritmi.
 *
 * Usare questa classe per poter calcolare i bioritmi di una persona in un dato
 * giorno.
 * Sviluppata inizialmente per GCC, pu essere utilizzata su qualsiasi compilatore
 * che supporti correttamente il C++ standard e la libreria standard
 *
 * /todo Impostare la compatibilit con BCB in modo da poter utilizzare anche i
 * tipi di data forniti dalle VCL
 */

class bioritmo
{
	public:
		//@{
		/**
		 * \name Costruttori
		 * \brief Funzioni costruttori della classe.
		 *
		 * Vengono usati per creare oggetti di questa classe
     	 */

		/** \brief Costruttore di default */
		bioritmo()
  		{
  			CFisico = -101;
    	}

    	/**
  	     * \brief Costruttore completo
  	     *
  	     * Si pu utilizzare per inizializzare i bioritmi alla creazione della
  	     * classe passandole le due date necessarie.
  	     * \param n data di nascita della persona
  	     * \param b data del giorno di cui si vuole il bioritmo
  	     */
		bioritmo(const QDate &n, const QDate &b);
		//@}

 		//@{
 		/**
 		 *
 		 * \name Impostazione date
 		 * \brief Funzioni per impostare le date della classe
 		 *
 		 * Vengono utilizzate per far sapere alla classe qual' la data di
 		 * nascita e di quale giorno si vuole sapere il bioritmo.
 		 */

 		/**
 		 * \brief Impostazione data bioritmo tramite array.
 		 *
 		 * Inizializza la data utilizzando un array, verificando se questa 
 		 * valida.
 		 * \param date Array contenente i 3 valori, giorno, mese, anno
 		 * @return 0 se la data  valida, il valore di ritorno di t_date in caso
 		 * contrario.
 		 */
	    bool set_bio_date(int date[]);

	    /**
	     * \brief Impostazione data bioritmo tramite tre interi.
	     *
	     * Inizializza la data utilizzando tre variabili intere, verificando se
	     * queste rappresentano una data valida.
	     * \param d Giorno del mese, deve essere valido per il mese inserito.
	     * \param m Mese dell'anno, deve essere compreso tra 1 e 12
	     * \param y Anno rappresentato a 4 cifre, e superiore a 1582.
	     * @return 0 se la data  valida, il valore di ritorno di t_date in caso
	     * contrario.
	     */
		bool set_bio_date(int d, int m, int y);

		/**
         * \brief Imposta la data del bioritmo da un t_date inizializzato
         *
         * Funzione veloce per poter inizializzare la data del bioritmo
         * con un t_date inizializzato
         * \param date la variabile t_date che contiene la data
         */
		inline void set_bio_date(QDate date)
		{
			Bioritmo = date;
			made = false;
		} // set_bio_date

 		/**
 		 * \brief Impostazione data di nascita tramite array.
 		 *
 		 * Inizializza la data utilizzando un array, verificando se questa 
 		 * valida.
 		 * \param date Array contenente i 3 valori, giorno, mese, anno
 		 * @return 0 se la data  valida, il valore di ritorno di t_date in caso
 		 * contrario.
 		 */
		bool set_bir_date(int date[]);

	    /**
	     * \brief Impostazione data di nascita tramite tre interi.
	     *
	     * Inizializza la data utilizzando tre variabili intere, verificando se
	     * queste rappresentano una data valida.
	     * \param d Giorno del mese, deve essere valido per il mese inserito.
	     * \param m Mese dell'anno, deve essere compreso tra 1 e 12
	     * \param y Anno rappresentato a 4 cifre, e superiore a 1582.
	     * @return 0 se la data  valida, il valore di ritorno di t_date in caso
	     * contrario.
	     */
		bool set_bir_date(int d, int m, int y);

		/**
         * \brief Imposta la data di nascita da un t_date inizializzato.
         *
         * Funzione veloce per poter inizializzare la data di nascita
         * con un t_date inizializzato
         */
		inline void set_bir_date(QDate date)
		{
			Birthdate = date;
			made = false;
		} // set_bir_date
		//@}

		/**
		 * \brief Funzione di calcolo dei bioritmi.
		 *
		 * Esegue il calcolo dei bioritmi e salva nei membri privati della classe
         * i valori dei cicli in modo da poterli leggere.
         */
		void make_bioritmi();

		//@{
		/**
		 * \name Lettura cicli
		 * \brief Funzioni per la lettura dei cicli generati.
		 *
		 * Utilizzate queste funzioni per poter leggere i valori calcolati.\n
		 * Per sicurezza leggete prima il valore di CFisico in modo da sapere
		 * se sono stati calcolati o no.
		 */

		/**
		 * \brief Ciclo Fisico.
		 *
		 * @return Il valore del ciclo fisico calcolato per la data richiesta, o
		 * -101 (valore che non pu essere calcolato), se non  stato possibile
		 * calcolare i bioritmi.
		 */
		inline int cFisico()
		{
			if ( ! made ) make_bioritmi();
			return CFisico;
		} // cFisico

		/**
		 * \brief Ciclo Emotivo.
		 *
		 * @return Il valore del ciclo emotivo calcolato per la data richiesta.
		 */
		inline int cEmotivo()
		{
			if ( ! made ) make_bioritmi();
			return CEmotivo;
		} // cEmotivo

		/**
		 * \brief Ciclo Inellettuale.
		 *
		 * @return Il valore del ciclo intellettuale calcolato per la data
         * richiesta.
		 */
		inline int cIntellettuale()
		{
			if ( ! made ) make_bioritmi();
			return CIntellettuale;
		} // cIntellettuale

		/**
		 * \brief Valore Medio.
		 *
		 * @return Il valore medio dei cicli bioritmici calcolati per la data
		 * richiesta.
		 */
		inline int cMedio()
		{
			return (cFisico() + cEmotivo() + cIntellettuale()) / 3;
		} // cMedio

		/**
		 * \brief Estrazione dei valori
		 *
		 * Permette di leggere con una sola chiamata i valori di tutti i cicli
		 * e del valore medio calcolati per la data richiesta.
		 * \param F Variabile in cui salvare il ciclo fisico.
		 * \param E Variabile in cui salvare il ciclo emotivo.
		 * \param I Variabile in cui salvare il ciclo intellettuale.
		 * \param M Variabile in cui salvare il valore medio dei cicli.
		 */
		inline void valori(int &F, int &E, int &I, int &M)
		{
			if ( ! made ) make_bioritmi();
			F = cFisico();
			E = cEmotivo();
			I = cIntellettuale();
			M = (F + E + I) / 3;
		} // cicli

		//@}

	private:
		QDate Bioritmo;
          ///< Data del bioritmo
		QDate Birthdate;
		  ///< Data di nascita

		int CFisico;
		  ///< Variabile intera contenente il valore dei ciclo fisico
		int CEmotivo;
		  ///< Variabile intera contenente il valore dei ciclo emotivo
		int CIntellettuale;
		  ///< Variabile intera contenente il valore dei ciclo intellettuale

		bool made;
};

#endif

